/*
 * [ExportHolidaysToHTML.java]
 *
 * Summary: Export list of holidays to HTML for summary display.
 *
 * Copyright: (c) 2008-2011 Roedy Green, Canadian Mind Products, http://mindprod.com
 *
 * Licence: This software may be copied and used freely for any purpose but military.
 *          http://mindprod.com/contact/nonmil.html
 *
 * Requires: JDK 1.5+
 *
 * Created with: JetBrains IntelliJ IDEA IDE http://www.jetbrains.com/idea/
 *
 * Version History:
 *  1.0 2008-12-03 initial version
 */
package com.mindprod.holidays;

import com.mindprod.common11.BigDate;
import com.mindprod.common11.Misc;

import java.io.BufferedWriter;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collections;

import static com.mindprod.entities.EntifyStrings.entifyHTML;
import static java.lang.System.out;

/**
 * Export list of holidays to HTML for summary display.
 *
 * @author Roedy Green, Canadian Mind Products
 * @version 1.0 2008-12-03 initial version
 * @since 2008
 */
public final class ExportHolidaysToHTML
    {
    // ------------------------------ CONSTANTS ------------------------------

    /**
     * Default year to display a list of holidays for.
     */
    private static final int thisYear = Misc.thisYear();

    /**
     * where to send the exported HTML description of the holidays
     */
    private static final String EXPORT_TO = "E:/mindprod/jgloss/include/holidays.html";

    /**
     * A delegate object that implements the HolInfo abstract class. That lets us execute code for a number of different
     * holidays given only the name of the class that computes them. We dynamically load the HollInfo classes with
     * Class.forName.
     */
    private static HolInfo[] holidayDelegates;

    /**
     * holidays in order for this year.
     */
    private static ArrayList<HolInfo> sortedHolidays;

    // -------------------------- STATIC METHODS --------------------------

    /**
     * export list of all holidays as html for this year
     *
     * @throws IOException if trouble writing file.
     */
    private static void exportHolidays() throws IOException
        {
        sortedHolidays = new ArrayList<HolInfo>( holidayDelegates.length );
        for ( HolInfo h : holidayDelegates )
            {
            // extract just holidays celebrated this year.
            if ( h != null && h.when( thisYear, false ) != BigDate.NULL_ORDINAL )
                {
                sortedHolidays.add( h );
                }
            }
        Collections.sort( sortedHolidays, new HolidaySort( thisYear, false ) );
        // count how many holidays in each month
        int[] holidaysInMonth = new int[ 12 ];
        for ( HolInfo h : sortedHolidays )
            {
            holidaysInMonth[ new BigDate( h.when( thisYear, false ) ).getMM() - 1 ]++;
            }
        int springSummerRows = 0;
        for ( int i = 0; i < 6; i++ )
            {
            if ( holidaysInMonth[ i ] > springSummerRows )
                {
                springSummerRows = holidaysInMonth[ i ];
                }
            }
        int fallWinterRows = 0;
        for ( int i = 6; i < 12; i++ )
            {
            if ( holidaysInMonth[ i ] > fallWinterRows )
                {
                fallWinterRows = holidaysInMonth[ i ];
                }
            }
        String[][] matrix = new String[ Math.max( springSummerRows, fallWinterRows ) ][ 12 ];
        {
        int row = 0;
        int prevMonth = -1;
        for ( HolInfo h : sortedHolidays )
            {
            int month = new BigDate( h.when( thisYear, false ) ).getMM();
            if ( month != prevMonth )
                {
                row = 0;
                prevMonth = month;
                }
            matrix[ row++ ][ month - 1 ] = h.getName();
            }
        }
        FileOutputStream fos = new FileOutputStream( EXPORT_TO, false /* append */ );
        OutputStreamWriter osw = new OutputStreamWriter( fos );
        BufferedWriter bw = new BufferedWriter( osw, 4096 /* buffsize in chars */ );
        PrintWriter prw = new PrintWriter( bw, false /* auto flush on println */ );
        // export SpringSummer header
        prw.println( "<table class=\"borderless\" summary=\"holidays\">\n" +
                     "<!-- DO NOT EDIT. generated by com.mindprod.holidays.ExportHolidaysToHTML program. -->" +
                     "<thead>\n" +
                     "<tr>\n" +
                     "<th>January</th>\n" +
                     "<th>February</th>\n" +
                     "<th>March</th>\n" +
                     "<th>April</th>\n" +
                     "<th>May</th>\n" +
                     "<th>June</th>\n" +
                     "</tr>\n" +
                     "</thead>\n" +
                     "<tbody>" );
        // export springSummer data
        for ( int row = 0; row < springSummerRows; row++ )
            {
            prw.println( "<tr>" +
                         "<!-- DO NOT EDIT. generated by com.mindprod.holidays.ExportHolidaysToHTML program. -->" );
            for ( int month = 0; month < 6; month++ )
                {
                if ( matrix[ row ][ month ] == null )
                    {
                    prw.println( "<td>&nbsp;</td>" );
                    }
                else
                    {
                    prw.print( "<td>" );
                    prw.print( entifyHTML( matrix[ row ][ month ] ) );
                    prw.println( "</td>" );
                    }
                }
            prw.println( "</tr>" );
            }
        // export FallWinter header
        prw.println( "<tr>\n" +
                     "<!-- DO NOT EDIT. generated by com.mindprod.holidays.ExportHolidaysToHTML program. -->" +
                     "<th>July</th>\n" +
                     "<th>August</th>\n" +
                     "<th>September</th>\n" +
                     "<th>October</th>\n" +
                     "<th>November</th>\n" +
                     "<th>December</th>\n" +
                     "</tr>" );
        // export FallWinter data
        for ( int row = 0; row < fallWinterRows; row++ )
            {
            prw.println( "<tr>" +
                         "<!-- DO NOT EDIT. generated by com.mindprod.holidays.ExportHolidaysToHTML program. -->" );
            for ( int month = 6; month < 12; month++ )
                {
                if ( matrix[ row ][ month ] == null )
                    {
                    prw.println( "<td>&nbsp;</td>" );
                    }
                else
                    {
                    prw.print( "<td>" );
                    prw.print( entifyHTML( matrix[ row ][ month ] ) );
                    prw.println( "</td>" );
                    }
                }
            prw.println( "</tr>" );
            }
        // wrap up
        prw.println( "</tbody>\n" +
                     "<!-- DO NOT EDIT. generated by com.mindprod.holidays.ExportHolidaysToHTML program. -->" +
                     "</table>" );
        prw.close();
        }

    /**
     * get list of holidays from properties file: com.mindprod.holidays.Holiday.properties which lives in the jar. class
     * names are case sensitive. file looks like this: <br /> Christmas=yes<br /> GroundhogDay=no <br /> ... <br /> where
     * yes=include no=ignore
     */
    private static void findHolidays()
        {
        String[][] result = null;
        try
            {
            // look in jar, and on classpath for
            // com.mindprod.holidays.Holiday.properties
            final InputStream fis = ExportHolidaysToHTML.class
                    .getResourceAsStream( "Holiday.properties" );
            result = Misc.loadProperties( fis );
            // we now key=value pairs
            }
        catch ( IOException oops )
            {
            out.println( oops
                         + " Problem accessing Holiday.properties file." );
            System.exit( 1 );
            }
        // in pairs className=yes/no
        int length = result[ 0 ].length;
        int j = 0;
        // keep just the holidays marked yes.
        // in the holidayDelegate array.
        holidayDelegates = new HolInfo[ length ];
        for ( int i = 0; i < length; i++ )
            {
            try
                {
                if ( result[ 1 ][ i ].equalsIgnoreCase( "yes" ) )
                    {
                    holidayDelegates[ j++ ] = ( HolInfo ) ( Class
                            .forName( "com.mindprod.holidays."
                                      + result[ 0 ][ i ] )
                            .newInstance() );
                    }
                }
            catch ( Exception oops )
                {
                out.println( oops
                             + " Bug in Holiday.properties or class file for "
                             + result[ 0 ][ i ] );
                System.exit( 1 );
                }
            }// end for
        }

    // --------------------------- main() method ---------------------------

    /**
     * Allow this applet to run as as application as well.
     *
     * @param args command line arguments ignored.
     *
     * @throws IOException if cannot load or export all the holidays.
     */
    public static void main( String args[] ) throws IOException
        {
        findHolidays();
        exportHolidays();
        }// end main
    }
